require 'noteCells'

function showDelayColumns(trackIndex)

  local track = renoise.song():track(trackIndex)

  if track.type == 1 then
    track.delay_column_visible = true
  end    
end

function getNumberOfStepsInSelection()

  local selectionInPattern = renoise.song().selection_in_pattern
  local startLine = selectionInPattern.start_line
  local endLine = selectionInPattern.end_line

  return (endLine-startLine+1)*256
end

function getNumberOfPopulatedLines(trackIndexValue)

  local numberOfPopulatedLines = 0

  for trackIndex in pairs(noteCells) do
  
    if trackIndex == trackIndexValue then
      local previousLineIndex = -1
      local numberOfNoteCellsInTrack = #noteCells[trackIndex]
      for i = 1, numberOfNoteCellsInTrack do
  
        local noteCell = noteCells[trackIndex][i]
        
        if noteCell.line ~= previousLineIndex then
          numberOfPopulatedLines = numberOfPopulatedLines + 1
          previousLineIndex = noteCell.line  
        end
      end
    end
  end
  
  return numberOfPopulatedLines
end

function distributeNotesEvenlyByLine()

  local numberOfStepsInSelection = getNumberOfStepsInSelection()

  for trackIndex in pairs(noteCells) do
   
    local numberOfPopulatedLines = getNumberOfPopulatedLines(trackIndex)
    local populatedLineIndex = 0
  
    if numberOfPopulatedLines > 0 then
      
      local previousLineIndex = -1
      local numberOfNoteCellsInTrack = #noteCells[trackIndex]
      for i = 1, numberOfNoteCellsInTrack do
  
        local noteCell = noteCells[trackIndex][i]
        
        if noteCell.line ~= previousLineIndex then
          populatedLineIndex = populatedLineIndex + 1
          previousLineIndex = noteCell.line  
        end
        
        local numberOfSteps = math.floor((populatedLineIndex-1)*(numberOfStepsInSelection/numberOfPopulatedLines) + 0.5)
        noteCell:setNotePosition(numberOfSteps)
        
        if noteCell.delay > 0 then
          showDelayColumns(trackIndex)
        end
      end
    end
  end
end

function distributeNotesEvenlyByColumn()

  local numberOfStepsInSelection = getNumberOfStepsInSelection()

  for trackIndex in pairs(noteCells) do
  
      local numberOfNoteCellsInTrack = #noteCells[trackIndex]
      for i = 1, numberOfNoteCellsInTrack do
  
        local noteCell = noteCells[trackIndex][i]
        local numberOfSteps = math.floor((i-1)*(numberOfStepsInSelection/numberOfNoteCellsInTrack) + 0.5)
        noteCell:setNotePosition(numberOfSteps)
        
        if noteCell.delay > 0 then
          showDelayColumns(trackIndex)
        end
      end
  end
end

function placeSelectedNotesEvenlyByLine()

  collectNoteCellsFromSelection()
  distributeNotesEvenlyByLine()
  pasteNoteCells()
end


function placeSelectedNotesEvenlyByColumn()

  collectNoteCellsFromSelection()
  distributeNotesEvenlyByColumn()
  pasteNoteCells()
end

renoise.tool():add_menu_entry {
  name = "Pattern Editor:Selection:place selected notes evenly",
  invoke = function() placeSelectedNotesEvenlyByLine() end
}

renoise.tool():add_keybinding {
  name = "Pattern Editor:Selection:place selected notes evenly",
  invoke = function() placeSelectedNotesEvenlyByLine() end
}

renoise.tool():add_menu_entry {
  name = "Pattern Editor:Selection:place selected notes evenly (by column)",
  invoke = function() placeSelectedNotesEvenlyByColumn() end
}

renoise.tool():add_keybinding {
  name = "Pattern Editor:Selection:place selected notes evenly (by column)",
  invoke = function() placeSelectedNotesEvenlyByColumn() end
}
